# Please update related files if you modified these attributes

# TODO:

# Append char(s) to abbr:
#
# Two words can be separated if someone want convert all abbr. to lowercase
# e.g., HTTPProtocol ---(convert to lowercase)---> httpprotocol
#       (it is difficult to identify the two words)
#
#       HTTPProtocol --- (append '_' to abbr) ---> http_Protocol
#                    ---(convert to lowercase)---> http_protocol


# How can we find an abbr:
#
#   good:
#       HTTP[^A-Za-z]*Response (HTTP)
#
#   bad, the outputs of these names may be wrong below:
#       httpresponse (can't be identified)
#       HTTPresponse (HTT)


# Priority 0 is highest priority
#   origin                    Get_HTTPResponse-Code

# alphabet
#   upper_case                GET_HTTPRESPONSE-CODE
#   lower_case                get_httpresponse-code
#   camel_case                get_HttpResponse-Code
#   pascal_case               Get_HttpResponse-Code

# abbr
#   ignore_abbr               Get_HTTPResponse-Code (with pascal_case)
#                             get_HTTPresponse-code (with lower_case)

# sep
#   dash_to_underscore        Get_HTTPResponse_Code
#   underscore_to_dash        Get-HTTPResponse-Code

FORMATS = {
    'LETTER_CASE': [
        'upper_case',
        'lower_case',
        'camel_case',
        'pascal_case',
        'ignore',
    ],
    'SEP': [
        'dash_to_underscore',
        'underscore_to_dash',
    ],
    'MODE': [  # set 'SUFFIX' to 'c'
        'add',  # 'abccc' -> 'abcccc'
        'remove',  # 'abccc' -> 'abcc'
        'force_add',  # 'abccc' -> 'abc'
        'force_remove',  # 'abccc' -> 'ab'
        'ignore',
    ],
}

# Default settings
SETTINGS = {
    "CHECK_DIRS": [],
    "INCLUDE_FILES": [],
    "INCLUDE_DIRS": [],
    "IGNORE_FILES": [],
    "IGNORE_DIRS": [],
    "FILE_FORMATS": {
        "LETTER_CASE": "ignore",
        "SEP": [],
    },
    "DIR_FORMATS": {
        "LETTER_CASE": "ignore",
        "SEP": [],
    },
    "FILE_PREFIX_MODE": "ignore",
    "FILE_SUFFIX_MODE": "ignore",
    "FILE_PREFIX": "",
    "FILE_SUFFIX": "",
    "DIR_PREFIX_MODE": "ignore",
    "DIR_SUFFIX_MODE": "ignore",
    "DIR_PREFIX": "",
    "DIR_SUFFIX": "",
}



import os
import shutil


class ArchieveManager():
    """
    Strategy:
        1. file rename first
        2. deeper first

    :type path_pairs: list of tuple, e.g., [(src, dst)]
    """

    def rename(self, path_pairs):
        """
        :return: pairs which doesn't be renamed
        """
        error_pairs = []
        file_pairs, dir_pairs = (
            self._separate_file_dir_from_path_pair(path_pairs))

        error_pairs.extend(self._rename(file_pairs))
        error_pairs.extend(self._rename(dir_pairs))

        return error_pairs

    def _rename(self, path_pairs):
        """
        :return: pairs which doesn't be renamed
        """
        error_pairs = []
        path_pairs = self._sort_path_pair(path_pairs, reverse=True)

        for src, dst in path_pairs:
            try:
                shutil.move(src, dst)
            except Exception:
                error_pairs.append([src, dst])

        return error_pairs

    def gen_revert_path_pairs(self, path_pairs):
        # Only implement for file/dir under same original directory
        """
        TODO:
        Find more elegant and efficient way to convert files/directories
        which upper hierarchy path have been renamed.

        this function must be called before rename functions,
        because we need to separate files/directories first.

        When we want to rollback names,
        we need different strategy to rename those be renamed files.
        The deeper files/directories which have been renamed must be first.
        e.g.,
            rename:
                /change/upper -> /change/new_upper
                /change/upper/lower -> /change/new_upper/new_lower
            revert:
                /change/new_upper/new_lower -> /change/new_upper/lower
                /change/new_upper -> /change/upper
        """
        file_pairs, dir_pairs = (
            self._separate_file_dir_from_path_pair(path_pairs))
        file_pairs = self._sort_path_pair(file_pairs)
        dir_pairs = self._sort_path_pair(dir_pairs)
        revert_path_pairs = []
        renamed_mapping = {}

        for pairs in [dir_pairs, file_pairs]:
            for src, dst in pairs:
                # replace dirname
                # if any hierarchical dirname has changed
                renamed_dirname = os.path.dirname(dst)
                dirname_parts = renamed_dirname.split(os.sep)
                # ['/root/to/path', '/root/to', '/root']
                dirname_powerset = []
                for r in range(len(dirname_parts) + 1, 1, -1):
                    dirname_powerset.append(os.sep.join(dirname_parts[:r]))
                for dn in dirname_powerset:
                    if dn in renamed_mapping:
                        renamed_dn = renamed_mapping[dn]
                        renamed_dirname = (
                            renamed_dn + renamed_dirname[len(dn):])
                        break

                renamed_dst = os.sep.join(
                    [renamed_dirname, os.path.basename(dst)])
                renamed_src = os.sep.join(
                    [renamed_dirname, os.path.basename(src)])
                revert_path_pairs.append((renamed_dst, renamed_src))

                renamed_mapping[src] = renamed_dst

        return revert_path_pairs

    def _separate_file_dir_from_path_pair(self, path_pairs):
        file_pairs = []
        dir_pairs = []

        for pair in path_pairs:
            if os.path.isfile(pair[0]):
                file_pairs.append(pair)
            elif os.path.isdir(pair[0]):
                dir_pairs.append(pair)

        return file_pairs, dir_pairs

    def _sort_path_pair(self, path_pairs, **kwargs):
        reverse = kwargs.get('reverse', False)

        path_pairs = path_pairs[:]
        counted_path_pairs = []
        for pair in path_pairs:
            counted_path_pairs.append(
                [pair[0], pair[1], len(pair[0].split(os.sep))])
        counted_path_pairs.sort(
            key=lambda p: p[2] if os.path.isdir(p[0]) else p[2] - 1,
            reverse=reverse)

        sorted_ = []
        for pair in counted_path_pairs:
            sorted_.append((pair[0], pair[1]))

        return sorted_



import os
import json
import datetime
import sys
from namanager.core import Namanager
from namanager.archieve_manager import ArchieveManager
from namanager.logger import logger
import namanager.enums as enums
import namanager.util as util


def raiser(condition, msg):
    if not condition:
        raise Exception(msg)  # pragma: no cover


def test_writing_permission(**kwargs):
    """
    param dirname:
    type dirname: str
    param required: default True
    type required: True, False
    param error_msg:
    type error_msg: str
    """

    try:
        dirname = kwargs.get('dirname', os.getcwd())
        if dirname[-1] != os.sep:
            dirname += os.sep
        # test directory is exists or not and raise
        os.path.realpath(dirname)
        logger().info('dirname: {0}'.format(dirname))

        filename = ''.join([dirname, 'test_file'])
        while os.path.exists(filename):
            filename += '_'

        with open(filename, 'w') as f:
            f.write('test...')
            logger().info('successfully wrote: {0}.'.format(filename))

        with open(filename, 'r') as f:
            f.read()
            logger().info('successfully read: {0}.'.format(filename))

        os.remove(filename)

    except Exception as e:
        print(kwargs.get('error_msg', ''))
        required = kwargs.get('required', True)
        if required:
            raise e


class Driver():
    def __init__(self):
        self._result = {'errors': []}
        self._exit_code = 0

    @property
    def result(self):
        return self._result

    @property
    def exit_code(self):
        return self._exit_code

    def import_settings(self, settings_file):
        settings_json = {}
        try:
            with open(settings_file, 'r') as s:
                settings_json = json.loads(s.read())
        except Exception as e:
            file_not_found = False
            if sys.version_info[0] >= 3:
                if isinstance(e, FileNotFoundError):  # noqa: F821
                    file_not_found = True
            elif isinstance(e, IOError):  # noqa: F821
                file_not_found = True
            if file_not_found:
                self._result['errors'].append(
                    'File: {0} not found'.format(settings_file))
            else:
                raise e

        raiser(isinstance(settings_json, dict), 'settings must be dict.')

        return settings_json

    def get_src_dst_pair(self, error_info):
        # we need to move this function to/into a better place
        src_dst_pair = []

        for e in error_info:
            src_dst_pair.append([e['actual'], e['expect']])

        return src_dst_pair

    def get_bak_filename(self, **kwargs):
        prefix = kwargs.get('prefix', '')
        when = kwargs.get('when', '{:%Y%m%d%H%M%S}'.format(
            datetime.datetime.now()))

        return prefix + when + '.bak'

    def find_recent_backup_files(self, **kwargs):
        dirname = kwargs.get('dirname', os.getcwd())
        backup_files = []
        for dirpath, dirs, files in os.walk(dirname):
            for f in files:
                if f.startswith('namanager_rename_'):
                    backup_files.append(f)
            break

        return backup_files

    def revert(self, **kwargs):
        REVERT_FILE = kwargs.get('revert_file', None)
        REVERT_LAST = kwargs.get('revert_last', False)
        if REVERT_FILE is None or REVERT_LAST:
            backup_files = self.find_recent_backup_files()
            if len(backup_files) == 1:
                REVERT_FILE = backup_files[0]
            elif len(backup_files) > 1:
                if REVERT_LAST:
                    backup_files.sort(reverse=True)
                    REVERT_FILE = backup_files[0]
                else:
                    self._result['errors'].append(
                        'There are so many backup files, please specify file.')
            elif len(backup_files) == 0:
                self._result['errors'].append(
                    'No backup file are detected, please specify file.')

        try:
            am = ArchieveManager()
            with open(REVERT_FILE, 'r') as f:
                am.rename(json.loads(f.read()))
        except Exception as e:
            self._result['errors'].append(e)

    def generate_init_settings(self, **kwargs):
        INIT_PATH = kwargs.get('init_path', os.getcwd())
        settings = json.dumps(enums.SETTINGS, indent=4, sort_keys=True)
        settings_file = (os.path.join(INIT_PATH, 'settings.json')
                         if os.path.isdir(INIT_PATH)
                         else INIT_PATH)
        if os.path.exists(settings_file):
            self.result['errors'].append(
                'File existed. Cannot output settings file to {0}.\n'.format(
                    settings_file))
            return
        try:
            with open(settings_file, 'w') as f:
                f.write(settings)
        except Exception as e:
            self.result['errors'].append(
                'Cannot output settings file to {0}.\n'
                'Error: {1}'.format(
                    settings_file, e))

    def check(self, **kwargs):
        settings_json = kwargs.get('settings_json', enums.SETTINGS)
        COUNT = kwargs.get('count', False)
        FMT = kwargs.get('fmt', 'json')
        PRETTY_DUMP = kwargs.get('pretty_dump', False)
        self._result['unexpected_pairs'] = [] if FMT == 'nodump' else ''

        for d in settings_json['CHECK_DIRS']:
            if not util.isdir_casesensitive(d):
                continue

            checker = Namanager(settings_json)
            checker.check(d)

            if checker.error_info:
                unexpected_pairs = checker.error_info
                if COUNT:
                    self._result['errors'].append(
                        'In folder {0} :'.format(os.path.realpath(d)))
                    self._result['errors'].append(
                        'FAILED (error={0})'.format(checker.error_info_count))
            else:
                unexpected_pairs = []

            if FMT == 'readable':
                s = ""
                for pair in checker.get_dict(unexpected_pairs):
                    s += 'expect: {0}\n'.format(pair['expect'])
                    s += 'actual: {0}\n\n'.format(pair['actual'])
                self._result['unexpected_pairs'] += s
            elif FMT == 'json':
                self._result['unexpected_pairs'] += (
                    checker.get_json(unexpected_pairs, PRETTY_DUMP))
            elif FMT == 'xml':
                self._result['unexpected_pairs'] += (
                    checker.get_xml(unexpected_pairs, PRETTY_DUMP))
            elif FMT == 'nodump':
                self._result['unexpected_pairs'] += (
                    checker.get_dict(unexpected_pairs))

    def rename_backup(self, rename_pairs, **kwargs):
        am = ArchieveManager()
        RENAME_BACKUP = kwargs.get('rename_backup', True)
        logger().info('backup flag: {0}'.format(RENAME_BACKUP))

        if RENAME_BACKUP:
            RENAME_BACKUP_PATH = kwargs.get('rename_backup_path', os.getcwd())
            if os.path.isdir(RENAME_BACKUP_PATH):
                RENAME_BACKUP_PATH = os.sep.join([
                    RENAME_BACKUP_PATH,
                    self.get_bak_filename(prefix='namanager_rename_')])
            logger().info('backup path: {0}'.format(RENAME_BACKUP_PATH))
            test_writing_permission(
                dirname=os.path.dirname(RENAME_BACKUP_PATH))
            revert_pairs = am.gen_revert_path_pairs(rename_pairs)
            self._result['rename_backup_name'] = RENAME_BACKUP_PATH
            with open(RENAME_BACKUP_PATH, 'w') as f:
                f.write(json.dumps(revert_pairs, indent=4, sort_keys=True))
                logger().info('successfully wrote backup: {0}.'.format(
                    RENAME_BACKUP_PATH))

    def rename(self, rename_pairs, **kwargs):
        RENAME_RECOVER = kwargs.get('rename_recover', False)
        am = ArchieveManager()
        rename_pairs = self.get_src_dst_pair(rename_pairs)
        self.rename_backup(rename_pairs, **kwargs)

        error_pairs = am.rename(rename_pairs)

        if error_pairs:
            # TODO: output more information
            if RENAME_RECOVER:
                # try to directly revert all paths
                recover_pairs = am.gen_revert_path_pairs(rename_pairs)
                am.rename(recover_pairs)
                self._result['errors'].append("Failed to rename (Recovered).")

            else:
                self._result['errors'].append('Some paths can not be renamed.')

    def entry(self, **kwargs):
        REQUIRED = kwargs.get('required', False)
        VERSION = kwargs.get('version', False)
        INIT = kwargs.get('init', False)
        REVERT = kwargs.get('revert', False)
        RENAME = kwargs.get('rename', False)

        if VERSION:
            import namanager
            print(namanager.__version__)

        elif INIT:
            self.generate_init_settings(**kwargs)

        elif RENAME:
            SETTINGS = kwargs.get('settings', False)
            if SETTINGS:
                kwargs['settings_json'] = self.import_settings(SETTINGS)
            kwargs['fmt'] = 'nodump'
            self.check(**kwargs)
            self.rename(self._result['unexpected_pairs'], **kwargs)

        elif REVERT:
            self.revert(**kwargs)

        else:
            SETTINGS = kwargs.get('settings', False)
            if SETTINGS:
                kwargs['settings_json'] = self.import_settings(SETTINGS)
            self.check(**kwargs)
            print(self._result['unexpected_pairs'])

            if REQUIRED and self._result['unexpected_pairs']:
                self._exit_code = 1

        if self._result['errors']:
            for e in self._result['errors']:
                print(e)
            if REQUIRED:
                self._exit_code = 1



import os
import re
import json
import dicttoxml
from xml.dom.minidom import parseString
import namanager.util as util
import namanager.enums as enums
from namanager.logger import logger


class Namanager():
    def __init__(self, settings={}):
        self.init_settings()
        self.load_settings(settings)
        self.verify_setting_type()

        self._error_info = []
        self._error_info_count = 0

    def init_settings(self):
        # Please update `enums` if you modified these attributes
        self._FILE_FORMATS = (
            enums.SETTINGS['FILE_FORMATS'])
        self._FILE_SEP = (
            enums.SETTINGS['FILE_FORMATS']['SEP'])
        self._FILE_LETTER_CASE = (
            enums.SETTINGS['FILE_FORMATS']['LETTER_CASE'])
        self._INCLUDE_FILES = (
            enums.SETTINGS['INCLUDE_FILES'])
        self._IGNORE_FILES = (
            enums.SETTINGS['IGNORE_FILES'])

        self._DIR_FORMATS = (
            enums.SETTINGS['DIR_FORMATS'])
        self._DIR_SEP = (
            enums.SETTINGS['DIR_FORMATS']['SEP'])
        self._DIR_LETTER_CASE = (
            enums.SETTINGS['DIR_FORMATS']['LETTER_CASE'])
        self._INCLUDE_DIRS = (
            enums.SETTINGS['INCLUDE_DIRS'])
        self._IGNORE_DIRS = (
            enums.SETTINGS['IGNORE_DIRS'])

    def _convert_os_sep_of_str_in_list(self, strlist):
        # needs to test
        converted_strlist = []
        for s in strlist:
            converted_strlist.append(s.replace('/', os.sep))
        return converted_strlist

    def verify_setting_type(self):
        errors = []
        settings = [
            {'s': self._FILE_FORMATS,
             't': type(enums.SETTINGS['FILE_FORMATS'])},
            {'s': self._FILE_SEP,
             't': type(enums.SETTINGS['FILE_FORMATS']['SEP'])},
            {'s': self._FILE_LETTER_CASE,
             't': type(enums.SETTINGS['FILE_FORMATS']['LETTER_CASE'])},
            {'s': self._INCLUDE_FILES,
             't': type(enums.SETTINGS['INCLUDE_FILES'])},
            {'s': self._IGNORE_FILES,
             't': type(enums.SETTINGS['IGNORE_FILES'])},

            {'s': self._DIR_FORMATS,
             't': type(enums.SETTINGS['DIR_FORMATS'])},
            {'s': self._DIR_SEP,
             't': type(enums.SETTINGS['DIR_FORMATS']['SEP'])},
            {'s': self._DIR_LETTER_CASE,
             't': type(enums.SETTINGS['DIR_FORMATS']['LETTER_CASE'])},
            {'s': self._INCLUDE_DIRS,
             't': type(enums.SETTINGS['INCLUDE_DIRS'])},
            {'s': self._IGNORE_DIRS,
             't': type(enums.SETTINGS['IGNORE_DIRS'])},

            {'s': self._FILE_PREFIX_MODE,
             't': type(enums.SETTINGS['FILE_PREFIX_MODE'])},
            {'s': self._FILE_SUFFIX_MODE,
             't': type(enums.SETTINGS['FILE_SUFFIX_MODE'])},
            {'s': self._FILE_PREFIX,
             't': type(enums.SETTINGS['FILE_PREFIX'])},
            {'s': self._FILE_SUFFIX,
             't': type(enums.SETTINGS['FILE_SUFFIX'])},

            {'s': self._DIR_PREFIX_MODE,
             't': type(enums.SETTINGS['DIR_PREFIX_MODE'])},
            {'s': self._DIR_SUFFIX_MODE,
             't': type(enums.SETTINGS['DIR_SUFFIX_MODE'])},
            {'s': self._DIR_PREFIX,
             't': type(enums.SETTINGS['DIR_PREFIX'])},
            {'s': self._DIR_SUFFIX,
             't': type(enums.SETTINGS['DIR_SUFFIX'])},
        ]

        for setting in settings:
            if not isinstance(setting['s'], setting['t']):
                errors.append("Type of {0} ({1}) must be {2}.".format(
                    util.name(setting['s'], self.__dict__),
                    setting['s'],
                    setting['t']))

        cases = [self._FILE_LETTER_CASE, self._DIR_LETTER_CASE]
        for case in cases:
            if case not in enums.FORMATS['LETTER_CASE']:
                errors.append("the case: {0} ({1}) must be one of {2}.".format(
                    util.name(case, self.__dict__),
                    case,
                    enums.FORMATS['LETTER_CASE']))

        modes = [self._FILE_PREFIX_MODE,
                 self._FILE_SUFFIX_MODE,
                 self._DIR_PREFIX_MODE,
                 self._DIR_SUFFIX_MODE]
        for mode in modes:
            if mode not in enums.FORMATS['MODE']:
                errors.append("the mode: {0} ({1}) must be one of {2}.".format(
                    util.name(mode, self.__dict__),
                    mode,
                    enums.FORMATS['MODE']))

        if errors:
            err_str = '\n'
            for error in errors:
                err_str += error + '\n'
            raise TypeError(err_str)

    def load_settings(self, settings={}):
        # Please update `enums` if you modified these attributes
        self._FILE_FORMATS = settings.get(
            'FILE_FORMATS', enums.SETTINGS['FILE_FORMATS'])
        self._FILE_SEP = self._FILE_FORMATS.get(
            'SEP', enums.SETTINGS['FILE_FORMATS']['SEP'])
        self._FILE_LETTER_CASE = str(
            self._FILE_FORMATS.get(
                'LETTER_CASE', enums.SETTINGS['FILE_FORMATS']['LETTER_CASE']))
        self._INCLUDE_FILES = self._convert_os_sep_of_str_in_list(
            settings.get('INCLUDE_FILES', enums.SETTINGS['INCLUDE_FILES']))
        self._IGNORE_FILES = self._convert_os_sep_of_str_in_list(
            settings.get('IGNORE_FILES', enums.SETTINGS['IGNORE_FILES']))

        self._DIR_FORMATS = settings.get(
            'DIR_FORMATS', enums.SETTINGS['DIR_FORMATS'])
        self._DIR_SEP = self._DIR_FORMATS.get(
            'SEP', enums.SETTINGS['DIR_FORMATS']['SEP'])
        self._DIR_LETTER_CASE = str(
            self._DIR_FORMATS.get(
                'LETTER_CASE', enums.SETTINGS['DIR_FORMATS']['LETTER_CASE']))
        self._INCLUDE_DIRS = self._convert_os_sep_of_str_in_list(
            settings.get('INCLUDE_DIRS', enums.SETTINGS['INCLUDE_DIRS']))
        self._IGNORE_DIRS = self._convert_os_sep_of_str_in_list(
            settings.get('IGNORE_DIRS', enums.SETTINGS['IGNORE_DIRS']))

        self._FILE_PREFIX_MODE = (
            settings.get('FILE_PREFIX_MODE',
                         enums.SETTINGS['FILE_PREFIX_MODE']))
        self._FILE_SUFFIX_MODE = (
            settings.get('FILE_SUFFIX_MODE',
                         enums.SETTINGS['FILE_SUFFIX_MODE']))
        self._FILE_PREFIX = (
            settings.get('FILE_PREFIX', enums.SETTINGS['FILE_PREFIX']))
        self._FILE_SUFFIX = (
            settings.get('FILE_SUFFIX', enums.SETTINGS['FILE_SUFFIX']))

        self._DIR_PREFIX_MODE = (
            settings.get('DIR_PREFIX_MODE', enums.SETTINGS['DIR_PREFIX_MODE']))
        self._DIR_SUFFIX_MODE = (
            settings.get('DIR_SUFFIX_MODE', enums.SETTINGS['DIR_SUFFIX_MODE']))
        self._DIR_PREFIX = (
            settings.get('DIR_PREFIX', enums.SETTINGS['DIR_PREFIX']))
        self._DIR_SUFFIX = (
            settings.get('DIR_SUFFIX', enums.SETTINGS['DIR_SUFFIX']))

    @property
    def error_info(self):
        logger().debug('error_info:')
        logger().info(self._error_info)
        return self._error_info

    @property
    def error_info_count(self):
        logger().debug('error_info_count')
        logger().info(self._error_info_count)
        return self._error_info_count

    @property
    def file_formats(self):
        logger().debug('file_formats')
        logger().info(self._FILE_FORMATS)
        return self._FILE_FORMATS

    @property
    def dir_formats(self):
        logger().debug('dir_formats')
        logger().info(self._DIR_FORMATS)
        return self._DIR_FORMATS

    @property
    def include_files(self):
        logger().debug('include_files')
        logger().info(self._INCLUDE_FILES)
        return self._INCLUDE_FILES

    @property
    def include_dirs(self):
        logger().debug('include_dirs')
        logger().info(self._INCLUDE_DIRS)
        return self._INCLUDE_DIRS

    @property
    def ignore_files(self):
        logger().debug('ignore_files')
        logger().info(self._IGNORE_FILES)
        return self._IGNORE_FILES

    @property
    def ignore_dirs(self):
        logger().debug('ignore_dirs')
        logger().info(self._IGNORE_DIRS)
        return self._IGNORE_DIRS

    @property
    def file_prefix_mode(self):
        logger().debug('file_prefix_mode')
        logger().info(self._FILE_PREFIX_MODE)
        return self._FILE_PREFIX_MODE

    @property
    def file_suffix_mode(self):
        logger().debug('file_suffix_mode')
        logger().info(self._FILE_SUFFIX_MODE)
        return self._FILE_SUFFIX_MODE

    @property
    def file_prefix(self):
        logger().debug('file_prefix')
        logger().info(self._FILE_PREFIX)
        return self._FILE_PREFIX

    @property
    def file_suffix(self):
        logger().debug('file_suffix')
        logger().info(self._FILE_SUFFIX)
        return self._FILE_SUFFIX

    @property
    def dir_prefix_mode(self):
        logger().debug('dir_prefix_mode')
        logger().info(self._DIR_PREFIX_MODE)
        return self._DIR_PREFIX_MODE

    @property
    def dir_suffix_mode(self):
        logger().debug('dir_suffix_mode')
        logger().info(self._DIR_SUFFIX_MODE)
        return self._DIR_SUFFIX_MODE

    @property
    def dir_prefix(self):
        logger().debug('dir_prefix')
        logger().info(self._DIR_PREFIX)
        return self._DIR_PREFIX

    @property
    def dir_suffix(self):
        logger().debug('dir_suffix')
        logger().info(self._DIR_SUFFIX)
        return self._DIR_SUFFIX

    @property
    def file_sep(self):
        logger().debug('file_sep')
        logger().info(self._FILE_SEP)
        return self._FILE_SEP

    @property
    def file_letter_case(self):
        logger().debug('file_letter_case')
        logger().info(self._FILE_LETTER_CASE)
        return self._FILE_LETTER_CASE

    @property
    def dir_sep(self):
        logger().debug('dir_sep')
        logger().info(self._DIR_SEP)
        return self._DIR_SEP

    @property
    def dir_letter_case(self):
        logger().debug('dir_letter_case')
        logger().info(self._DIR_LETTER_CASE)
        return self._DIR_LETTER_CASE

    def _is_string_matching(self, string, re_match_list=[]):
        if string:
            for pattern in re_match_list:
                if re.search(pattern, string) is not None:
                    return True
        return False

    def _get_walk(self, root):
        return [path_info for path_info in os.walk(os.path.realpath(root))]

    def _get_root_in_walk(self, walk):
        root = ''
        if walk:
            root = walk[0][0]
            for (dirpath, dirs, files) in walk:
                if len(dirpath.split(os.sep)) < len(root.split(os.sep)):
                    root = dirpath
        return root

    def _divide_full_part_path_patterns(self, re_patterns):
        full_path_patterns = []
        part_path_patterns = []

        for pattern in re_patterns:
            if pattern.rfind(os.sep) == -1:
                part_path_patterns.append(pattern)
            else:
                full_path_patterns.append(pattern)

        return full_path_patterns, part_path_patterns

    def _divide_file_and_dir_name_of_patterns(self, re_patterns):
        separated_patterns = []

        for pattern in re_patterns:
            latest_sep_index = pattern.rfind(os.sep)
            dirname_pattern = pattern[:latest_sep_index] + '$'
            filename_pattern = '^' + pattern[latest_sep_index + 1:]

            separated_patterns.append({
                "dirname": dirname_pattern,
                "filename": filename_pattern
            })

        return separated_patterns

    def _include_re_patterns_of_files_in_walk(self, re_patterns, walk,
                                              root=None):
        if root is None:
            root = self._get_root_in_walk(walk)

        filtered_walk = []
        within, without = (
            self._divide_full_part_path_patterns(re_patterns))
        within = self._divide_file_and_dir_name_of_patterns(within)

        for (dirpath, dirs, files) in walk:
            filtered = []

            if without:
                for filename in files:
                    if self._is_string_matching(filename, without):
                        filtered.append(filename)

            if within:
                rel_dirpath = dirpath[len(root):] if dirpath != root else '/'
                for pattern in within:
                    if not self._is_string_matching(
                       rel_dirpath, [pattern['dirname']]):
                        continue

                    for filename in files:
                        if self._is_string_matching(
                           filename, [pattern['filename']]):
                            filtered.append(filename)

            # at least one file be included
            if filtered:
                filtered_walk.append((
                    dirpath, dirs, filtered))

        return filtered_walk

    def _ignore_re_patterns_of_files_in_walk(self, re_patterns, walk,
                                             root=None):
        include_walk = self._include_re_patterns_of_files_in_walk(
            re_patterns, walk, root)
        filtered_walk = []

        for (dirpath, dirs, files) in walk:
            include_dirpath = ()

            for (dp, dn, fn) in include_walk:
                if dp == dirpath:
                    include_dirpath = (dp, dn, fn)

            if include_dirpath:
                filtered_files = []
                for filename in files:
                    if filename not in include_dirpath[2]:
                        filtered_files.append(filename)
                if filtered_files:
                    filtered_walk.append((dirpath, dirs, filtered_files))
            else:
                filtered_walk.append((dirpath, dirs, files))

        return filtered_walk

    def _include_re_patterns_of_dirs_in_walk(self, re_patterns, walk,
                                             root=None):
        if root is None:
            root = self._get_root_in_walk(walk)

        filtered_walk = []
        for (dirpath, dirs, files) in walk:
            rel_dirpath = dirpath[len(root):]
            if rel_dirpath == '':
                rel_dirpath = '/'
            if self._is_string_matching(rel_dirpath, re_patterns):
                filtered_walk.append((dirpath, dirs, files))

        return filtered_walk

    def _ignore_re_patterns_of_dirs_in_walk(self, re_patterns, walk,
                                            root=None):
        include_walk = self._include_re_patterns_of_dirs_in_walk(
            re_patterns, walk, root)
        filtered_walk = []

        for (dirpath, dirs, files) in walk:
            include_dirpath = ()

            for (dp, dn, fn) in include_walk:
                if dp == dirpath:
                    include_dirpath = (dp, dn, fn)

            if not include_dirpath:
                filtered_walk.append((dirpath, dirs, files))

        return filtered_walk

    def _get_file_walk(self, walk):
        if self.include_files:
            walk = self._include_re_patterns_of_files_in_walk(
                self.include_files, walk)
        if self.ignore_files:
            walk = self._ignore_re_patterns_of_files_in_walk(
                self.ignore_files, walk)

        return walk

    def _get_dir_walk(self, walk):
        if self.include_dirs:
            walk = self._include_re_patterns_of_dirs_in_walk(
                self.include_dirs, walk)
        if self.ignore_dirs:
            walk = self._ignore_re_patterns_of_dirs_in_walk(
                self.ignore_dirs, walk)

        return walk

    def check_file(self, root):
        root = os.path.realpath(root)
        walk = self._get_walk(root)
        walk = self._get_file_walk(walk)
        for dirpath, dirs, files in walk:
            for f in files:
                actual = f
                expect = self.get_expect_filename(f)

                if expect != actual:
                    self._error_info.append({
                        'expect': os.sep.join([dirpath, expect]),
                        'actual': os.sep.join([dirpath, actual])
                    })
                    self._error_info_count += len(expect)

    def check_dir(self, root):
        """
        Only check dirs/files path under root (excluded)
            root:     /root/path/to/dir
            dirpath:  /root/path/to/dir/SUBDIR
            settings: dir_format['letter_case'] = 'upper_case'

            OK: any part path of /root/path/to/dir will be never checked
        """
        root = os.path.realpath(root)
        walk = self._get_walk(root)
        walk = self._get_dir_walk(walk)
        for dirpath, dirs, files in walk:
            dirpath_dirname = os.path.dirname(dirpath)
            actual = os.sep + dirpath.split(os.sep)[-1]
            expect = self.get_expect_dirname(actual)

            if expect != actual:
                self._error_info.append({
                    'expect': "".join([dirpath_dirname, expect]),
                    'actual': "".join([dirpath_dirname, actual])
                })
                self._error_info_count += 1

    def get_extension(self, filename):
        extension = (''
                     if filename.find(r'\.') == -1
                     else '.' + filename.split(r'\.')[-1])
        return extension

    def get_filename_without_extension(self, filename):
        extension = self.get_extension(filename)
        return filename[:filename.rfind(extension)]

    def get_expect_filename(self, filename):
        name = self.get_filename_without_extension(filename)
        extension = self.get_extension(filename)

        name = util.convert_sep(name, self.file_sep)
        if self.file_letter_case != 'ignore':
            name = util.convert_sentence_to_case(
                name, self.file_letter_case)
        filename = self.modify_prefix_suffix_of_filename(name + extension)

        return filename

    def get_expect_dirname(self, dirname):
        dirname = util.convert_sep(dirname, self.dir_sep)
        if self.dir_letter_case != 'ignore':
            dirname = util.convert_sentence_to_case(
                dirname, self.dir_letter_case)
        dirname = self.modify_prefix_suffix_of_dirname(dirname)

        return dirname

    def modify_prefix_suffix_of_filename(self, filename):
        filename = self.get_name_with_prefix_by_mode(
            filename, self.file_prefix, self.file_prefix_mode)
        filename = self.get_name_with_suffix_by_mode(
            filename, self.file_suffix, self.file_suffix_mode)
        return filename

    def modify_prefix_suffix_of_dirname(self, dirname):
        dirname = self.get_name_with_prefix_by_mode(
            dirname, self.dir_prefix, self.dir_prefix_mode)
        dirname = self.get_name_with_suffix_by_mode(
            dirname, self.dir_suffix, self.dir_suffix_mode)
        return dirname

    def get_name_with_prefix_by_mode(self, name, prefix, mode):
        if mode == 'add':
            name = prefix + name
        elif mode == 'remove':
            if name[:len(prefix)] == prefix:
                name = name[len(prefix):]
        elif mode == 'force_add' or mode == 'force_remove':
            while name[:len(prefix)] == prefix:
                name = name[len(prefix):]
            if mode == 'force_add':
                name = prefix + name

        return name

    def get_name_with_suffix_by_mode(self, name, suffix, mode):
        return self.get_name_with_prefix_by_mode(
            name[::-1], suffix[::-1], mode)[::-1]

    def check(self, root):
        self.check_dir(root)
        self.check_file(root)

    def get_dict(self, data={}):
        return data

    def get_json(self, data={}, pretty_dump=False):
        if pretty_dump:
            return json.dumps(data, indent=4, sort_keys=True)
        else:
            return json.dumps(data)

    def get_xml(self, data={}, pretty_dump=False):
        xml = dicttoxml.dicttoxml(data)
        if pretty_dump:
            return parseString(xml).toprettyxml()
        else:
            return parseString(xml).toxml()



import inspect
import logging
import os


def logger():
    global _logger_instances
    # inspect.stack(1)[2]: caller infomations.
    caller_filename = inspect.stack(1)[2][1]
    if caller_filename not in _logger_instances:
        caller = caller_filename.split(os.sep)[-1]
        _logger_instances[caller_filename] = logging.getLogger(caller)
    return _logger_instances[caller_filename]


_logger_instances = {}



import setuptools
import os
import namanager

NAME = 'namanager'
VERSION = namanager.__version__
DESCRIPTION = (
    "A file or/and directory name manager which could determine"
    "names are/aren't expectable, "
    "and you could also automatically rename it."
)
LONG_DESCRIPTION = open(
     os.path.join(os.path.dirname(__file__), 'README.rst')).read()

LICENSE = 'GNU General Public License v3.0'
AUTHOR = 'Ernest Chang'
AUTHOR_EMAIL = 'iattempt.net@gmail.com'
MAINTAINER = (
    'Ernest Chang (https://github.com/iattempt) '
    'and Arnav Borborah (https://github.com/arnavb) '
)
MAINTAINER_EMAIL = (
    'iattempt.net@gmail.com'
    ', arnavborborah11@gmail.com'
)
URL = 'https://github.com/iattempt/namanager'
PACKAGES = {'namanager'}
PACKAGES = setuptools.find_packages(exclude=['tests'])
SCRIPTS = ['bin/namanager']
KEYWORDS = ['namanager', 'name', 'manager', 'checker', 'formatter', 'filename']

with open('requirements.txt') as f:
    INSTALL_REQUIRES = ([
        line if line[-1] != '\n' else line[:-1]
        for line in f if line and not line.startswith('- ')
    ])
PYTHON_REQUIRES = '>=2.7,!=3.0.*,!=3.1.*,!=3.2.*,'

setuptools.setup(
    name=NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    license=LICENSE,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    maintainer=MAINTAINER,
    maintainer_email=MAINTAINER_EMAIL,
    url=URL,

    keywords=KEYWORDS,
    classifiers=[
        'Topic :: Software Development',
        'Topic :: Utilities',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Operating System :: OS Independent',
        'Intended Audience :: Customer Service',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Environment :: Console',
        'Natural Language :: English',
    ],

    packages=PACKAGES,
    scripts=SCRIPTS,
    install_requires=INSTALL_REQUIRES,
    python_requires=PYTHON_REQUIRES,
)



import re
import os


def name(obj, callingLocals=locals()):
    name = None
    for k, v in list(callingLocals.items()):
        if v is obj:
            name = k
    return name


def gen_unique_str(string):
    u = '_'
    while u in string:
        u += '_'
    return u


def get_first_word(s):
    try:
        beg = s.find(re.search('[A-Za-z]', s).group(0)) if len(s) else -1
    except Exception:
        beg = -1
    end = beg + 1 if len(s) else 0
    while end < len(s):
        # e.g., HTTPResponse
        if (s[end - 1].isupper() and
            s[end].isupper() and
           (end + 1 != len(s) and s[end + 1].islower())):
            break
        # e.g., HttpResponse or Http_response
        if s[end - 1].islower() and not s[end].islower():
            break
        # e.g., HTTP* or HTTP_
        if not s[end].isalpha():
            break

        end += 1

    return (s[beg:end], beg, end)


def get_words(string, include_non_letter=True):
    words = []
    while string != '':
        if include_non_letter and not string[0].isalpha():
            first_letter = re.search('[a-zA-Z]', string)
            if first_letter:
                i = string.find(first_letter.group(0))
                words.append(string[:i])
                string = string[i:]
            else:
                words.append(string)
                string = ''
        else:
            tpl = get_first_word(string)
            if tpl[0]:
                string = string[tpl[2]:]
                words.append(tpl[0])
            else:
                string = ''

    return [w for w in words if w != '']


def convert_sep(string, cases):
    for case in cases:
        if case == 'dash_to_underscore':
            string = string.replace('-', '_')
        elif case == 'underscore_to_dash':
            string = string.replace('_', '-')
        else:
            raise KeyError(
                "key {0} is not found in the set of existing keys.".format(
                    case))

    return string


def convert_word_to_case(word, case):
    """
    Camel case is insensible for single word.

    User can convert all words to pascal case
    and then change first letter to lowercase
    if one want to convert words to camel case.
    """

    if case == 'upper_case':
        word = word.upper()
    elif case == 'lower_case':
        word = word.lower()
    elif case == 'pascal_case':
        word = word[0].upper() + word[1:].lower()
    else:
        raise KeyError(
            "key {0} is not found in the set of existing keys.".format(case))

    return word


def convert_sentence_to_case(sentence, case):
    return ''.join(convert_words_to_case(get_words(sentence), case))


def convert_words_to_case(words, case):
    """
    This function assuming that
    all words are well separated and
    not included empty string:
        Ok:
            ['_*&', 'Http', 'protocol', '#$%']
        Wrong:
            ['_*&h', 'ttp', 'protocol', '#$%']
    """

    converted_words = []

    if case == 'camel_case':
        first_word_occured = False
        for w in words:
            word = convert_word_to_case(w, 'pascal_case')
            if not first_word_occured and word[0].isalpha():
                word = word[0].lower() + word[1:]
                first_word_occured = True
            converted_words.append(word)

    elif case in ['upper_case', 'lower_case', 'pascal_case']:
        for w in words:
            converted_words.append(convert_word_to_case(w, case))

    else:
        raise KeyError(
            "key {0} is not found in the set of existing keys.".format(case))

    return converted_words


def get_full_path(path):
    return os.path.realpath(os.path.relpath(path))


def isfile_casesensitive(path):
    path = get_full_path(path)
    if not os.path.isfile(path):
        return False   # exit early
    directory, filename = os.path.split(path)
    return filename in os.listdir(directory)


def isdir_casesensitive(path):
    path = get_full_path(path)
    if not os.path.isdir(path):
        return False   # exit early
    directory, dirname = os.path.split(path)
    return dirname in os.listdir(directory)
